/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import java.util.Iterator;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.AbstractPage;
import it.eng.console.hibernate.pagination.CriteriaHibernatePage;
import it.eng.console.hibernate.pagination.QueryHibernatePage;
import it.eng.console.util.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.tracing.TracerSingleton;

public class MessagesListAction extends ProcessInstanceListFilteredAction {
	protected int _page;
	protected String _processo = null;
	protected String _idEndPoint = null;
	protected String _stato = null;
	protected String _dataInizio= null;
	protected String _dataFine = null;
	protected String _tipoOrdinamento = null;
	protected List _relevantAttributeProcessList = null;
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		int page = getRequestPage(serviceRequest, sessionContainer, Constants.PAGE_MESSAGES);

		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		
		//save info for pagination
		SourceBean messagesBeanList = new SourceBean("LIST");
		saveParametersForPagination(messagesBeanList);
		
		// recupero (se ci sono) e salvo attributi rilevanti di processo 
		saveDynamicInput(serviceRequest, messagesBeanList);

		//call ProcessInstanceBO
		List messagesList = null;
		Query query = null;
		AbstractPage hibernatePage = null;
		query = ProcessInstanceBO.getMessagesList(Long.parseLong(_idEndPoint), _stato, _dataInizio, _dataFine, _tipoOrdinamento, _relevantAttributeProcessList);
		//pagination
		hibernatePage = new QueryHibernatePage(query, page, Constants.sizeMessagesList);
		messagesList = hibernatePage.getList();
		
		messagesBeanList.setAttribute("MESSAGES", messagesList);
		serviceResponse.setAttribute(messagesBeanList);
	}
	
	protected void loadParametersFromRequest(SourceBean request) {
		_processo = (String)request.getAttribute(Constants.LBL_PROCESS_ID);
		_idEndPoint = (String)request.getAttribute(Constants.LBL_ID_COMPONENT);
		_stato = (String)request.getAttribute(Constants.LBL_STATO);
		_dataInizio = (String)request.getAttribute(Constants.LBL_DATA_INIZIO);
		_dataFine = (String) request.getAttribute(Constants.LBL_DATA_FINE);
		_tipoOrdinamento = (String)request.getAttribute(Constants.LBL_TIPO_ORDINAMENTO);
	}
	
	private void saveParametersForPagination(SourceBean messagesBeanList) throws SourceBeanException {
		try {
			if(_processo != null)
				messagesBeanList.setAttribute(Constants.LBL_PROCESS_ID, _processo);
			if(_idEndPoint != null)
				messagesBeanList.setAttribute(Constants.LBL_ID_COMPONENT, _idEndPoint);
			if(_stato != null)
				messagesBeanList.setAttribute(Constants.LBL_STATO, _stato);
			if(_dataInizio != null)
				messagesBeanList.setAttribute(Constants.LBL_DATA_INIZIO, _dataInizio);
			if(_dataFine != null)
				messagesBeanList.setAttribute(Constants.LBL_DATA_FINE, _dataFine);
			if(_tipoOrdinamento != null)
				messagesBeanList.setAttribute(Constants.LBL_TIPO_ORDINAMENTO, _tipoOrdinamento);
		} catch (SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "MessagesListAction::saveParametersForPagination: ", sbe);
		}
	}

	private void saveDynamicInput(SourceBean serviceRequest, SourceBean messagesBeanList){
		List requestParameters = serviceRequest.getContainedAttributes();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if (attribute.getKey().contains(Constants.PREFIX_PROCESS_ATTRIBUTE )
				&& !attribute.getValue().equals(""))
				try {
					messagesBeanList.setAttribute(attribute);
				}catch(SourceBeanException sbe) {
					//TODO
				}
		}
	}
}
