/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.bo;

import it.eng.console.hibernate.util.HibernateSessionFactory;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.console.util.DateFormatter;
import it.eng.serviceregistry.IServiceRegistry;
import it.eng.serviceregistry.JAXRServiceRegistryImpl;
import it.eng.serviceregistry.ServiceRegistryException;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spagosoa.metadb.base.MepStateConstants;
import it.eng.spagosoa.metadb.base.ServiceAssemblyStateConstants;
import it.eng.spagosoa.metadb.model.Attribute;
import it.eng.spagosoa.metadb.model.AttributeIter;
import it.eng.spagosoa.metadb.model.InstanceSa;
import it.eng.spagosoa.metadb.model.JournalInstanceService;
import it.eng.spagosoa.metadb.model.LogJournal;
import it.eng.spagosoa.metadb.model.ServiceAssembly;

import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.xml.registry.JAXRException;
import javax.xml.registry.infomodel.Service;

import org.hibernate.CacheMode;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

public class ProcessInstanceBO {
	public static Query getProcessInstanceList() throws HibernateException{
		return getProcessInstanceList(Constants.ALL_PROCESS, Constants.ALL_ITER, null);
	}
	
	public static Query getProcessInstanceList(long processId) throws HibernateException {
		return getProcessInstanceList(processId, Constants.ALL_ITER, null);
	}

	public static Query getProcessInstanceListByIterType(long iterType, List instanceIter) throws HibernateException {
		return getProcessInstanceList(Constants.ALL_PROCESS, iterType, instanceIter);
	}
	
	public static Query getProcessInstanceList(long processId, long idIterType, List instanceIter) throws HibernateException {
		return getProcessInstanceList(processId, idIterType, null, null, null, null, null, null, instanceIter, null);

	}
	
	public static Criteria getProcessInstanceList(String taxonomyName, String taxonomyValue) throws Exception{
		IServiceRegistry serviceRegistry = null;
		Criteria criteria = null;
		try {
			serviceRegistry = new JAXRServiceRegistryImpl();
			serviceRegistry.initRegistry();
			List serviceList = serviceRegistry.getServicesClassifiedByTaxonomy(taxonomyName, taxonomyValue);
			int indexService;
			Service service;
			String uddiName, uddiOrganization;
			
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(InstanceSa.class, "instanceSa")
					.createCriteria("serviceAssembly", Criteria.INNER_JOIN);
			if(serviceList.size() > 0) {
				Disjunction orExpressions = Restrictions.disjunction();
				for(indexService=0; indexService<serviceList.size(); indexService++) {
					service = (Service)serviceList.get(indexService);
					uddiName = service.getName().getValue();				
					uddiOrganization = service.getProvidingOrganization().getName().getValue();
					orExpressions.add(Expression.and(Expression.eq("uddiOrg", uddiOrganization), Expression.eq("uddiName", uddiName)));
				}
				criteria.add(orExpressions);
			} else
				criteria.add(Expression.eq("serviceAssembly.idAssembly", "-1"));
			
			criteria.addOrder(Order.asc("name"));
			criteria.addOrder(Order.desc("instanceSa.startdate"));
			//join per recuperare iter e istanza di iter 
			criteria.createCriteria("instanceSa.instanceIter", Criteria.LEFT_JOIN);
			
		} catch(ServiceRegistryException sre) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", sre);
			throw sre;
		} catch(JAXRException jax) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", jax);
			throw jax;
		} catch(Exception ex) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", ex);
			throw ex;
		} finally {
			serviceRegistry.finalizeRegistry();			
		}
		return criteria;
	}

	public static Query getProcessInstanceList(long process, long idIterType, String state, String startDate, String endDate, String org, String orderby, String orderType, List iterAttributes, List processAttributes) 
		throws HibernateException {			
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			//************ parsing date ************ 
			Date start = DateFormatter.toDate(startDate);
			Date end = DateFormatter.toDate(endDate);
			
			//*************  starting query ********************
			String hqlString = "SELECT new list(instanceSa.idInstanceSa, serviceAssembly.idAssembly, serviceAssembly.name, "; 
			if (idIterType == Constants.NO_ITER)
				hqlString += " '','',";
			else
				hqlString +=" iterType.name, "
						  + " ( SELECT istanceIterSub.uniqueIterInstanceKey FROM  it.eng.spagosoa.metadb.model.InstanceIter istanceIterSub "
						  + "  WHERE istanceIterSub.idIter = instanceSa.instanceIter.idIter ) as attributeValue, ";
		
			hqlString +=    " serviceAssembly.uddiOrg, instanceSa.startdate, instanceSa.enddate, instanceSa.state, instanceSa.idCorrelation)"
							+ " FROM it.eng.spagosoa.metadb.model.ServiceAssembly serviceAssembly," 
							+ " it.eng.spagosoa.metadb.model.InstanceSa instanceSa ";
			
			//********* join sull'istanza di iter *******
			if (idIterType == Constants.ALL_ITER) 
				hqlString +=  " LEFT OUTER JOIN instanceSa.instanceIter as instanceIter"
					+ " LEFT OUTER JOIN instanceIter.iterType as iterType";
			//iter selezionato
			else  if(idIterType != Constants.NO_ITER)				
				hqlString +=  " INNER JOIN instanceSa.instanceIter as instanceIter"
							+ " INNER JOIN instanceIter.iterType as iterType"
							+ " WITH iterType.idIterType = :idIterType";
			//*********** fine join sull'istanza di iter ********
			
			hqlString += " WHERE serviceAssembly.idAssembly = instanceSa.serviceAssembly.idAssembly";
			
			if(idIterType == Constants.NO_ITER)
				hqlString += " AND instanceSa.instanceIter IS NULL";
			if(process != Constants.ALL_PROCESS)
				hqlString += " AND serviceAssembly.idAssembly = " + process;
			else 
				hqlString += " AND serviceAssembly.saVersion = (SELECT max(serviceAssemblySubselect.saVersion)" 
					       + " FROM it.eng.spagosoa.metadb.model.ServiceAssembly serviceAssemblySubselect"
					       + " WHERE serviceAssemblySubselect.name = serviceAssembly.name)";
			if(state!=null && !state.equals(""))
				hqlString += " AND instanceSa.state = " + state;
			if(start!=null) 					
				hqlString += " AND instanceSa.startdate >= :start_date";
			if(end!=null) 						
				hqlString += " AND instanceSa.enddate <= :end_date";
			if(org!=null && !org.equals(""))
				hqlString += " AND serviceAssembly.uddiOrg = :org";
			
			
			//condizione relative agli attributi di istanza di iter 
			hqlString += createInstanceIterConditions(iterAttributes);	
			
			//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
			hqlString += createProcessRelevantDataConditions(processAttributes);
			
			//ricerca avanzata
			if(orderby!=null) {
				if(orderby.equalsIgnoreCase("DATA"))
					hqlString += " ORDER BY instanceSa.startdate " + orderType;
				else if(orderby.equalsIgnoreCase("OWNER"))
					hqlString += " ORDER BY serviceAssembly.uddiOrg " + orderType + ", instanceSa.startdate desc";
				else if(orderby.equalsIgnoreCase("PROCESS"))
					hqlString += " ORDER BY serviceAssembly.name " + orderType + ", instanceSa.startdate desc"; 
			}
			//ricerca senza aver specificato iter
			else if (idIterType == Constants.ALL_ITER || idIterType == Constants.NO_ITER)
				hqlString += " ORDER BY instanceSa.startdate DESC";
			else 
				hqlString += " ORDER BY instanceSa.instanceIter.idIter DESC, instanceSa.startdate DESC";
			query = session.createQuery(hqlString);
			if(idIterType != Constants.ALL_ITER && idIterType != Constants.NO_ITER)
				query.setLong("idIterType", (new Long(idIterType)).longValue());
			if(start!=null)
				query.setTimestamp("start_date", start);
			if(end!=null) 	
				query.setTimestamp("end_date", end);
			if(org!=null && !org.equals(""))
				query.setString("org", org);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getProcessInstanceList:: ", he);
			throw he;
		}	
		return query;
	}

	private static String createProcessRelevantDataConditions(List processAttributes) {
		String hqlString = "";
		if(processAttributes != null && processAttributes.size() > 0 ) {
			Iterator it = processAttributes.iterator();
			SourceBeanAttribute attribute = null;
			long idAttribute;
			String valueAttribute;
			int prefixIndex = Constants.PREFIX_PROCESS_ATTRIBUTE.length();
			while(it.hasNext()) {
				attribute = (SourceBeanAttribute)it.next();
				idAttribute = (new Long(attribute.getKey().substring(prefixIndex))).longValue();
				valueAttribute = (String)attribute.getValue();
				hqlString += " AND EXISTS ( "
					+" FROM  it.eng.spagosoa.metadb.model.RelevantData relevantDataSubselect,"
					+" it.eng.spagosoa.metadb.model.RelevantDataInstanceSa relevantDataInstanceSubselect"
					+" WHERE relevantDataInstanceSubselect.relevantData.idRelevantData = relevantDataSubselect.idRelevantData"
					+" AND relevantDataSubselect.attribute.idAttribute = " + idAttribute 
					+" AND relevantDataSubselect.value like '%" + valueAttribute + "%'"
					+" AND relevantDataInstanceSubselect.instanceSa.idInstanceSa = instanceSa.idInstanceSa)";
			}
		}
		return hqlString;
	}
	
	private static String createInstanceIterConditions(List iterAttributes){
		String hqlString = "";
		if(iterAttributes != null && iterAttributes.size() > 0 ) {
			Iterator it = iterAttributes.iterator();
			SourceBeanAttribute attribute = null;
			long idAttribute;
			String valueAttribute;
			int prefixIndex = Constants.PREFIX_ATTRIBUTE.length();
			while(it.hasNext()) {
				attribute = (SourceBeanAttribute)it.next();
				idAttribute = (new Long(attribute.getKey().substring(prefixIndex))).longValue();
				valueAttribute = (String)attribute.getValue();
				hqlString += " AND EXISTS ( "
							+" FROM  it.eng.spagosoa.metadb.model.RelevantData relevantDataSubselect,"
							+" it.eng.spagosoa.metadb.model.RelevantDataInstanceSa relevantDataInstance,"
							+" it.eng.spagosoa.metadb.model.AttributeIter attributeIter"
							+" WHERE relevantDataSubselect.attribute.idAttribute = " + idAttribute 
							+" AND relevantDataSubselect.value like '%" + valueAttribute + "%'"
							+" AND relevantDataSubselect.idRelevantData = relevantDataInstance.relevantData.idRelevantData"
							+" AND relevantDataInstance.instanceSa.idInstanceSa = instanceSa.idInstanceSa)";
			}
		}
		return hqlString;
	}
			
	public static List getProcessInstanceIdList(long process, long idIterType, String state, String startDate, String endDate, String org, String orderby, String orderType, List iterAttributes, List processAttributes) 
	throws HibernateException {			
	Query query = null;
	Session session = null;
	try {
		session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
		//************ parsing date ************ 
		Date start = DateFormatter.toDate(startDate);
		Date end = DateFormatter.toDate(endDate);
		
		//*************  starting query ********************
		String hqlString = "SELECT instanceSa.idInstanceSa" 
					+ " FROM ServiceAssembly serviceAssembly, InstanceSa instanceSa ";
		
		//********* join sull'istanza di iter *******
		if (idIterType == Constants.ALL_ITER) 
			hqlString +=  " LEFT OUTER JOIN instanceSa.instanceIter as instanceIter"
				+ " LEFT OUTER JOIN instanceIter.iterType as iterType";
		//iter selezionato
		else  if(idIterType != Constants.NO_ITER)				
			hqlString +=  " INNER JOIN instanceSa.instanceIter as instanceIter"
						+ " INNER JOIN instanceIter.iterType as iterType"
						+ " WITH iterType.idIterType = :idIterType";
		//*********** fine join sull'istanza di iter ********
		
		hqlString += " WHERE serviceAssembly.idAssembly = instanceSa.serviceAssembly.idAssembly";
		
		if(idIterType == Constants.NO_ITER)
			hqlString += " AND instanceSa.instanceIter IS NULL";
		if(process != Constants.ALL_PROCESS)
			hqlString += " AND serviceAssembly.idAssembly = " + process;
		else 
			hqlString += " AND serviceAssembly.saVersion = (SELECT max(serviceAssemblySubselect.saVersion)" 
				       + " FROM ServiceAssembly serviceAssemblySubselect"
				       + " WHERE serviceAssemblySubselect.name = serviceAssembly.name)";
		if(state!=null && !state.equals(""))
			hqlString += " AND instanceSa.state = " + state;
		if(start!=null) 					
			hqlString += " AND instanceSa.startdate >= :start_date";
		if(end!=null) 						
			hqlString += " AND instanceSa.enddate <= :end_date";
		if(org!=null && !org.equals(""))
			hqlString += " AND serviceAssembly.uddiOrg = :org";
		
		
		//condizione relative agli attributi di istanza di iter 
		hqlString += createInstanceIterConditions(iterAttributes);	
		
		//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
		hqlString += createProcessRelevantDataConditions(processAttributes);
		
		//ricerca avanzata
		if(orderby!=null) {
			if(orderby.equalsIgnoreCase("DATA"))
				hqlString += " ORDER BY instanceSa.startdate " + orderType;
			else if(orderby.equalsIgnoreCase("OWNER"))
				hqlString += " ORDER BY serviceAssembly.uddiOrg " + orderType + ", instanceSa.startdate desc";
			else if(orderby.equalsIgnoreCase("PROCESS"))
				hqlString += " ORDER BY serviceAssembly.name " + orderType + ", instanceSa.startdate desc"; 
		}
		//ricerca senza aver specificato iter
		else if (idIterType == Constants.ALL_ITER || idIterType == Constants.NO_ITER)
			hqlString += " ORDER BY instanceSa.startdate DESC";
		else 
			hqlString += " ORDER BY instanceSa.instanceIter.idIter DESC, instanceSa.startdate DESC";
		query = session.createQuery(hqlString);
		if(idIterType != Constants.ALL_ITER && idIterType != Constants.NO_ITER)
			query.setLong("idIterType", (new Long(idIterType)).longValue());
		if(start!=null)
			query.setTimestamp("start_date", start);
		if(end!=null) 	
			query.setTimestamp("end_date", end);
		if(org!=null && !org.equals(""))
			query.setString("org", org);
	} catch(HibernateException he) {
		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getProcessInstanceIdList:: ", he);
		throw he;
	}	
	return query.list();
}

	public static Query getMessagesList(long idEndPoint, String state, String startDate, String endDate, String orderType, List processAttributes) 
	throws HibernateException {			
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			//************ parsing date ************ 
			Date start = DateFormatter.toDate(startDate);
			Date end = DateFormatter.toDate(endDate);
			
			//*************  starting query ********************
			String hqlString = "SELECT new list(journalInstanceService.idJournal,"
							+ " journalInstanceService.instanceSa.idInstanceSa,"
							+ " journalInstanceService.mepState.idState,"
							+ " journalInstanceService.exchangeStart,"
							+ " journalInstanceService.exchangeEnd)" 
							+ " FROM JournalInstanceService journalInstanceService," 
							+ " InstanceSa instanceSa"
							+ " WHERE journalInstanceService.endpointByIdEndpointTarget.idEndpoint = :idEndpoint"
							+ " AND journalInstanceService.instanceSa.idInstanceSa = instanceSa.idInstanceSa";

			if(state!=null && !state.equals("")) 
				hqlString += " AND instanceSa.state = " + state;
			if(start!=null) 					
				hqlString += " AND journalInstanceService.exchangeStart >= :start_date";
			if(end!=null) 						
				hqlString += " AND journalInstanceService.exchangeStart <= :end_date";
			
			//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
			hqlString += createProcessRelevantDataConditions(processAttributes);
			
			hqlString += " ORDER BY journalInstanceService.exchangeStart " + orderType;
			query = session.createQuery(hqlString);
			query.setLong("idEndpoint", idEndPoint);
			if(start!=null)
				query.setTimestamp("start_date", start);
			if(end!=null) 	
				query.setTimestamp("end_date", end);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getMessagesList: ", he);
			throw he;
		}	
		return query;
	}

	public static List getMessagesIdList(long idEndPoint, String state, String startDate, String endDate, String orderType, List processAttributes) 
		throws HibernateException {			
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			//************ parsing date ************ 
			Date start = DateFormatter.toDate(startDate);
			Date end = DateFormatter.toDate(endDate);
			
			//*************  starting query ********************
			String hqlString = "SELECT journalInstanceService.idJournal"
							+ " FROM JournalInstanceService journalInstanceService, InstanceSa instanceSa"
							+ " WHERE journalInstanceService.endpointByIdEndpointTarget.idEndpoint = :idEndpoint"
							+ " AND journalInstanceService.instanceSa.idInstanceSa = instanceSa.idInstanceSa";

			if(state!=null && !state.equals("")) 
				hqlString += " AND instanceSa.state = " + state;
			if(start!=null) 					
				hqlString += " AND journalInstanceService.exchangeStart >= :start_date";
			if(end!=null) 						
				hqlString += " AND journalInstanceService.exchangeStart <= :end_date";
			
			//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
			hqlString += createProcessRelevantDataConditions(processAttributes);
			
			hqlString += " ORDER BY journalInstanceService.exchangeStart " + orderType;
			query = session.createQuery(hqlString);
			query.setLong("idEndpoint", idEndPoint);
			if(start!=null)
				query.setTimestamp("start_date", start);
			if(end!=null) 	
				query.setTimestamp("end_date", end);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getMessagesIdList: ", he);
			throw he;
		}	
		return query.list();
	}

	public static List getJournalInstanceService(long endPointId, long processInstanceId) 
	throws HibernateException {
		List journalInstanceService = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Criteria criteria = session.createCriteria(JournalInstanceService.class);
			criteria.add(Expression.eq("endpointByIdEndpointTarget.idEndpoint", endPointId));
			criteria.add(Expression.eq("instanceSa.idInstanceSa", processInstanceId));
			criteria.addOrder(Order.asc("exchangeStart"));
			journalInstanceService = criteria.list();			
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getJournalInstanceService(long endPointId, long processInstanceId):: ", he);
			throw he;
		} 	
		return journalInstanceService;
	}
	
	public static List getLogsJournal(long idJournal) throws HibernateException {
		List logsJournal = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Criteria criteria = session.createCriteria(LogJournal.class);
			criteria.add(Expression.eq("journalInstanceService.idJournal", idJournal));
			criteria.addOrder(Order.asc("datelog"));
			logsJournal = criteria.list();	
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getLogsJournal(long idJournal): ", he);
			throw he;
		} 		
		return logsJournal;
	}
	
	public static int getExecutionStateInstanceService(long endPointId, long processInstanceId) 
		throws HibernateException {
		if (ProcessBO.getEndPointDetail(endPointId).isIsStart())
			return ServiceAssemblyStateConstants.SA_DONE;
		List journalInstanceServiceList = getJournalInstanceService(endPointId, processInstanceId);
		int state;
		JournalInstanceService journalInstanceService = null;
		for (int index=0; index<journalInstanceServiceList.size(); index++) {
			journalInstanceService = (JournalInstanceService)journalInstanceServiceList.get(index);
			state = journalInstanceService.getMepState().getIdState();
			if(isFaultState(state))
				return ServiceAssemblyStateConstants.SA_FAULTED;
			if(isActiveState(state)) 				
				return ServiceAssemblyStateConstants.SA_ACTIVE;
		}
		return ServiceAssemblyStateConstants.SA_DONE;
	}
	
	public static boolean isFaultState(int state) {
		if(state == MepStateConstants.MEP_INONLY_FAULT
				|| state == MepStateConstants.MEP_INOPTIONALOUT_STEP1_FAULT
				|| state == MepStateConstants.MEP_INOPTIONALOUT_STEP2_FAULT
				|| state == MepStateConstants.MEP_INOUT_FAULT
				|| state == MepStateConstants.MEP_ROBUSTINONLY_STEP1_FAULT)
			return true;
		else 
			return false;
		
	}
	
	public static boolean isActiveState(int state) {
		if(state == MepStateConstants.MEP_INONLY_ACTIVE
				|| state == MepStateConstants.MEP_INOUT_ACTIVE
				|| state == MepStateConstants.MEP_INOPTIONALOUT_ACTIVE
				|| state == MepStateConstants.MEP_ROBUSTINONLY_ACTIVE
				|| state == MepStateConstants.MEP_INOUT_OUT
				|| state == MepStateConstants.MEP_INOPTIONALOUT_STEP1_OUT)
			return true;
		else 
			return false;
	}
	
	public static long getCountInstancesSa(long processId){
		long instanceSa = 0;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT count(instanceSa.idInstanceSa) " 	 
							+ " FROM InstanceSa instanceSa "								
							+ " WHERE instanceSa.serviceAssembly.idAssembly = :idAssembly";
			
			Query query = session.createQuery(hqlString);
			query.setLong("idAssembly", processId);
			
			instanceSa = (Long)query.list().get(0);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getInstanceSa:: ", he);
			throw he;
		} 		
		return instanceSa;
	}
		
	public static long getCountInstancesSa(long processId, long idIterType){
		long instanceSa = 0;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT count(instanceSa.idInstanceSa) " 	 
							+ " FROM it.eng.spagosoa.metadb.model.InstanceSa instanceSa";								
			hqlString += " WHERE instanceSa.serviceAssembly.idAssembly = :idAssembly";
			//iter selezionato
			if (idIterType != Constants.ALL_ITER && idIterType != Constants.NO_ITER) 				
				hqlString +=  " AND instanceSa.instanceIter.iterType.idIterType = :idIterType";
			if (idIterType == Constants.NO_ITER)
				hqlString +=  " AND instanceSa.instanceIter IS NULL";
			
			Query query = session.createQuery(hqlString);
			query.setLong("idAssembly", processId);
			if (idIterType != Constants.ALL_ITER && idIterType != Constants.NO_ITER)
				query.setLong("idIterType", idIterType);
			
			instanceSa = (Long)query.list().get(0);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getCountInstancesSa:: ", he);
			throw he;
		} 		
		return instanceSa;
	}
	
	public static String getAttributeName(long idAttribute) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Attribute attribute = (Attribute)session.get(Attribute.class, idAttribute);	
			return attribute.getName();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getAttributeName: ", he);
			throw he;
		}
	}
	
	public static int getStateInstanceProcess(long instanceId) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			InstanceSa instanceSa = (InstanceSa)session.get(InstanceSa.class, instanceId);	
			return instanceSa.getState();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getStateInstanceProcess: ", he);
			throw he;
		}
	}

	public static int getStateJournal(long idJournal) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			JournalInstanceService journal = (JournalInstanceService)session.get(JournalInstanceService.class, idJournal);	
			return journal.getMepState().getIdState();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getStateJournal: ", he);
			throw he;
		}
	}
	
	public static long getIdJournal(long idLog) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			LogJournal log = (LogJournal)session.get(LogJournal.class, idLog);	
			return log.getJournalInstanceService().getIdJournal();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getIdJournal: ", he);
			throw he;
		}
	}
	
	public static List getInstanceSummaryInfo(long idInstance) {
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			//*************  starting query ********************
			String hqlString = "SELECT new list(serviceAssembly.name, instanceSa.state, serviceAssembly.idAssembly, instanceSa.idCorrelation)"
							+ " FROM InstanceSa instanceSa, ServiceAssembly serviceAssembly "
							+ " WHERE instanceSa.idInstanceSa = :idInstance AND instanceSa.serviceAssembly = serviceAssembly";
			
			query = session.createQuery(hqlString);
			query.setLong("idInstance", idInstance);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getInstanceSummaryInfo: ", he);
			throw he;
		}	
		List summaryInfo = query.list();
		return (List)summaryInfo.get(0);				
	}
	
	
}
