/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.module;

import java.util.Iterator;
import java.util.List;

import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.module.AbstractModule;
import it.eng.spago.tracing.TracerSingleton;

public class ProcessInstanceListFilterModule extends AbstractModule {
	
	protected int _page;
	protected String _processo = null;
	protected String _version;
	protected String _stato = null;
	protected String _dataInizio= null;
	protected String _dataFine = null;
	protected String _owner = null;
	protected String _ordinamento = null;
	protected String _tipoOrdinamento = null;
	protected String _idIterType = null;
	protected String _restart = null;
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
	throws Exception {
		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		
		//save info for pagination
		SourceBean paginationInfo = new SourceBean("PAGINATION_INFO");
		saveParametersForPagination(paginationInfo);
		// recupero (se ci sono) parametri legati all'iter e all'istanza 
		// e (se ci sono) attributi rilevanti di processo 
		saveDynamicInput(serviceRequest, paginationInfo);
		
		serviceResponse.setAttribute(paginationInfo);
	}
	
	protected void loadParametersFromRequest(SourceBean request) {
		_processo = (String)request.getAttribute(Constants.LBL_PROCESS_ID);
		_version = (String)request.getAttribute(Constants.LBL_VERSION);
		_stato = (String)request.getAttribute(Constants.LBL_STATO);
		_dataInizio = (String)request.getAttribute(Constants.LBL_DATA_INIZIO);
		_dataFine = (String) request.getAttribute(Constants.LBL_DATA_FINE);
		_owner = (String)request.getAttribute(Constants.LBL_OWNER);
		_ordinamento = (String)request.getAttribute(Constants.LBL_ORDINAMENTO);
		_tipoOrdinamento = (String)request.getAttribute(Constants.LBL_TIPO_ORDINAMENTO);
		_idIterType = (String)request.getAttribute(Constants.LBL_SELECTED_ITER);
		_restart = (String)request.getAttribute(Constants.LBL_RESTART);
	}
	
	private void saveParametersForPagination(SourceBean paginationInfo) throws SourceBeanException {
		try {
			if(_processo != null)
				paginationInfo.setAttribute(Constants.LBL_PROCESS_ID, _processo);
			if(_version != null)
				paginationInfo.setAttribute(Constants.LBL_VERSION, _version);
			if(_stato != null)
				paginationInfo.setAttribute(Constants.LBL_STATO, _stato);
			if(_dataInizio != null)
				paginationInfo.setAttribute(Constants.LBL_DATA_INIZIO, _dataInizio);
			if(_dataFine != null)
				paginationInfo.setAttribute(Constants.LBL_DATA_FINE, _dataFine);
			if(_owner != null)
				paginationInfo.setAttribute(Constants.LBL_OWNER, _owner);
			if(_ordinamento != null)
				paginationInfo.setAttribute(Constants.LBL_ORDINAMENTO, _ordinamento);
			if(_tipoOrdinamento != null)
				paginationInfo.setAttribute(Constants.LBL_TIPO_ORDINAMENTO, _tipoOrdinamento);
			if(_idIterType!=null)
				paginationInfo.setAttribute(Constants.LBL_SELECTED_ITER, _idIterType);
			if(_restart!=null)
				paginationInfo.setAttribute(Constants.LBL_RESTART, _restart);
		} catch (SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredModule::saveParametersForPagination: ", sbe);
		}
	}

	private void saveDynamicInput(SourceBean serviceRequest, SourceBean paginationInfo){
		List requestParameters = serviceRequest.getContainedAttributes();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if ((attribute.getKey().contains(Constants.PREFIX_ATTRIBUTE ) 
					|| attribute.getKey().contains(Constants.PREFIX_PROCESS_ATTRIBUTE ) )
				&& !attribute.getValue().equals(""))
				try {
					paginationInfo.setAttribute(attribute);
				}catch(SourceBeanException sbe) {
					//TODO
				}
		}
	}
}