/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.module;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spagosoa.smx.components.restart.client.ModifiedMessage;
import it.eng.spagosoa.smx.components.restart.client.RestartResult;
import it.eng.spagosoa.smx.components.restart.client.RestartServiceProxy;

public class RestartMessagesModule extends MessagesListFilterModule {

	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		//get Parameters from request
		SourceBean paginationInfo = (SourceBean)serviceRequest.getAttribute("PAGINATION_INFO");
		loadParametersFromRequest(paginationInfo);
		//recupero (se ci sono) attributi rilevanti di processo 
		List processAttributes = getAttributeProcessInput(paginationInfo);

		//effettuo query e preparo input per WebService
		List messagesIdList = ProcessInstanceBO.getMessagesIdList(Long.parseLong(_idEndPoint), _stato, _dataInizio, _dataFine, _tipoOrdinamento, processAttributes);
		int messagesSize = messagesIdList.size();
		ModifiedMessage[] messagesList = new ModifiedMessage[messagesSize];
		int indexMessage;
		for (indexMessage=0; indexMessage<messagesSize; indexMessage++)
			messagesList[indexMessage] = new ModifiedMessage((Long)messagesIdList.get(indexMessage), null);  
			
		//call method to restart the messages
		RestartServiceProxy proxy = new RestartServiceProxy();
		String locationRestartWS = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.locationRestartWS");
		proxy.setEndpoint(locationRestartWS);
		RestartResult[] restartResult = null;
		try {
			// Restart messages
			restartResult = proxy.restartMultipleMessages(messagesList);
		} catch(RemoteException re) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RestartMessagesModule::service:: ", re);
			EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, "10001", "properties.console.messages");
			getErrorHandler().addError(warning);
			return;
		}
		
		long [] idOriginal = new long[restartResult.length];
		int indexOriginal;
		for(indexOriginal=0; indexOriginal<idOriginal.length; indexOriginal++)
			idOriginal[indexOriginal] = ((RestartResult)restartResult[indexOriginal]).getOriginalInstanceId();
		
		serviceResponse.setAttribute("idOriginal",idOriginal);
		serviceResponse.setAttribute("idRestarted",restartResult);
	}
	
	private List getAttributeProcessInput(SourceBean request){
		List requestParameters = request.getContainedAttributes();
		List relevantAttributeProcessList = new ArrayList();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if (attribute.getKey().contains(Constants.PREFIX_PROCESS_ATTRIBUTE ) && !attribute.getValue().equals(""))
				relevantAttributeProcessList.add(attribute);	
		}
		return relevantAttributeProcessList;
	}

}
