/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.endpoint.action;

import java.util.Iterator;
import java.util.Set;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.model.Property;
import it.eng.spagosoa.metadb.model.PropertyEndpoint;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.tracing.TracerSingleton;

public class EndPointDetailAction extends AbstractAction {

	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		String endPointId = (String)serviceRequest.getAttribute("endPointId");
		SourceBean endPointBean = createDetailBean(Long.parseLong(endPointId));
		SourceBean endPointPropertiesBean = createPropertiesBean(Long.parseLong(endPointId));
		endPointBean.setAttribute(endPointPropertiesBean);
		serviceResponse.setAttribute(endPointBean);
	}
	
	protected SourceBean createDetailBean(long endPointId) {
		SourceBean endPointBean = null;
		try {
			endPointBean = new SourceBean("DETAIL");
			Endpoint endPointDetail = ProcessBO.getEndPointDetail(endPointId);
			endPointBean.setAttribute("nome", endPointDetail.getName());
			endPointBean.setAttribute("descrizione", endPointDetail.getDescription());
			String componentName = (endPointDetail.getComponentName()!=null) ? endPointDetail.getComponentName() : "";
			endPointBean.setAttribute("service", componentName);
		}catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "EndPointDetailAction::createDetailBean:: ", sbe);
		} 
		return endPointBean;
	}
	
	protected SourceBean createPropertiesBean(long endPointId) {
		SourceBean endPointPropertiesBean = null;
		try {
			endPointPropertiesBean = new SourceBean("PROPERTIES");
			Set properties = ProcessBO.getEndPointProperties(endPointId);
			Iterator it = properties.iterator();
			Property property = null;
			SourceBean propertyBean = null;
			while(it.hasNext()) {
				property = ((PropertyEndpoint)it.next()).getProperty();
				propertyBean = new SourceBean("PROPERTY");
				propertyBean.setAttribute("name", property.getCode());
				propertyBean.setAttribute("value", property.getValue());
				endPointPropertiesBean.setAttribute(propertyBean);
			}
		}catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "EndPointDetailAction::createPropertiesBean:: ", sbe);
		}
		return endPointPropertiesBean;
	}

}
