/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.hibernate.util;

import it.eng.console.util.Constants;
import it.eng.spago.tracing.TracerSingleton;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.hibernate.SessionFactory;
import org.hibernate.StaleObjectStateException;

public class HibernateSessionRequestFilter implements Filter {

    private SessionFactory _sessionFactory;

    public void doFilter(ServletRequest request,
                         ServletResponse response,
                         FilterChain chain)
            throws IOException, ServletException {

        try {
            TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "HibernateSessionRequestFilter::doFilter:Starting a database transaction");
            _sessionFactory.getCurrentSession().beginTransaction();

            // Call the next filter (continue request processing)
            chain.doFilter(request, response);

            // Commit and cleanup
            TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "HibernateSessionRequestFilter::doFilter:Committing the database transaction");
            _sessionFactory.getCurrentSession().getTransaction().commit();

        } catch(StaleObjectStateException staleEx){
        	TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, staleEx.getMessage());
            throw staleEx;
        } catch(Throwable ex){
        	TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, ex.getMessage());
            try {
                if (_sessionFactory.getCurrentSession().getTransaction().isActive()) {
                	TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "HibernateSessionRequestFilter::doFilter:Trying to rollback database transaction after exception");
                	_sessionFactory.getCurrentSession().getTransaction().rollback();
                }
            } catch (Throwable rbEx) {
            	TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, rbEx.getMessage());
            }

            // Let others handle it... maybe another interceptor for exceptions?
            throw new ServletException(ex);
        }
    }

    public void init(FilterConfig filterConfig) throws ServletException {    	    
    	_sessionFactory = HibernateSessionFactory.getSessionFactory();    	
    }

    public void destroy() {}
}