/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.process.action;

import java.text.DateFormat;
import java.util.List;
import java.util.Locale;

import org.hibernate.Criteria;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.CriteriaHibernatePage;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spagosoa.metadb.model.ServiceAssembly;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.tracing.TracerSingleton;

public class ProcessListFilteredAction extends AbstractAction {
	private String _taxonomyName = null;
	private String _taxonomyValue = null;
	private String _idSelectedNode = null;			
	private String _iterType = null;	
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {		
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		int page = getRequestPage(serviceRequest, sessionContainer, Constants.PAGE_PROCESS);
		
		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		
		//save into response for pagination
		SourceBean processBeanList = new SourceBean("LIST");
		saveParametersForPagination(processBeanList);
	
		//call ProcessBO 
		List processList = null;
		Criteria criteria = null;
		CriteriaHibernatePage hibernatePage = null;
		if(_idSelectedNode!=null) 
			criteria = ProcessBO.getProcessList(_taxonomyName, _taxonomyValue);
		else if (_iterType!=null)
			criteria = ProcessBO.getProcessList((new Long(_iterType)).longValue());
		else
			criteria = ProcessBO.getProcessList();
		hibernatePage = new CriteriaHibernatePage(criteria, page, Constants.sizeProcessList);
		
		//pagination
//		if (page > hibernatePage.getLastPageNumber()){
//			hibernatePage.setPage(--page);
//			sessionContainer.setAttribute(Constants.PAGE, page);
//		} else if(page == Constants.LAST_PAGE_VALUE){
//			int lastPage = hibernatePage.getLastPageNumber();
//			hibernatePage.setPage(lastPage);
//			sessionContainer.setAttribute(Constants.PAGE, lastPage);
//		}
		processList = hibernatePage.getList();
		
		//create response
		ServiceAssembly process = null;		
		for(int indexProcess=0; indexProcess<processList.size(); indexProcess++) {
			process = (ServiceAssembly)processList.get(indexProcess);
			if (process!=null)
				processBeanList.setAttribute(createProcessBean(process, _iterType));				
		}
		serviceResponse.setAttribute(processBeanList);
	}
	
	protected int getRequestPage(SourceBean request, SessionContainer session, String label) {
		String pageFromRequest = (String)request.getAttribute(Constants.PAGE);
		int page = 0;
		if(pageFromRequest == null || pageFromRequest.equalsIgnoreCase(Constants.FIRST_PAGE))
			page = 0;
		else if(pageFromRequest.equalsIgnoreCase(Constants.PREV_PAGE)) {
			page = ((Integer)session.getAttribute(label)).intValue() - 1;
			page = (page < 0) ? 0 : page;
		} else if(pageFromRequest.equalsIgnoreCase(Constants.NEXT_PAGE)) { 
			page = ((Integer)session.getAttribute(label)).intValue() + 1;
		} else if(pageFromRequest.equalsIgnoreCase(Constants.LAST_PAGE)) 
			page = Constants.LAST_PAGE_VALUE;
		session.setAttribute(label, page);
		return page;
	}
	
	private void loadParametersFromRequest(SourceBean request) {
		_taxonomyName = (String)request.getAttribute(Constants.LBL_SELECTED_TAXONOMY);
		_taxonomyValue = (String)request.getAttribute(Constants.LBL_SELECTED_TAXONOMY_VALUE);
		_idSelectedNode = (String)request.getAttribute(Constants.LBL_SELECTED_NODE);			
		_iterType = (String)request.getAttribute(Constants.LBL_SELECTED_ITER);	
	}
	
	private void saveParametersForPagination(SourceBean processBeanList) throws SourceBeanException {
		try {
			if(_idSelectedNode!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_NODE, _idSelectedNode);
			if(_iterType!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_ITER, _iterType);
			if(_taxonomyName!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_TAXONOMY, _taxonomyName);
			if(_taxonomyValue!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_TAXONOMY_VALUE, _taxonomyValue);
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessListFilteredAction::saveParametersForPagination: ", sbe);
		}
	}
	
	protected SourceBean createProcessBean(ServiceAssembly process, String iterType) 
		throws SourceBeanException {		
		SourceBean item = null;
		long idAssembly;
		try {
			item = new SourceBean("ITEM");
			idAssembly = process.getIdAssembly();
			item.setAttribute("id", idAssembly);
			item.setAttribute("name", process.getName());				
						
			DateFormat formatter = DateFormat.getDateInstance(DateFormat.SHORT, Locale.ITALY);
			String dateTemp = formatter.format(process.getStartdate());					
			item.setAttribute("dataInizio", dateTemp);				
			dateTemp = (process.getEnddate() != null) ? formatter.format(process.getEnddate()) : "";					
			item.setAttribute("dataFine", dateTemp);
			
			String version = String.valueOf(process.getSaVersion());
			item.setAttribute("versione", version);	
			if(iterType == null)
				item.setAttribute("istanze", Long.toString(ProcessInstanceBO.getCountInstancesSa(idAssembly)));
			else
				item.setAttribute("istanze", Long.toString(ProcessInstanceBO.getCountInstancesSa(idAssembly, (new Long(iterType)).longValue())));
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessListAction::createProcessBean:: ", sbe);
			throw sbe;
		}
		
		return item;
	}
		
}
