/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.tag;

import it.eng.console.util.Constants;
import it.eng.spagosoa.metadb.base.ServiceAssemblyStateConstants;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

public class ProcessInstanceListTag extends TagSupport {
	private boolean _advancedSearch = false;
	private boolean _showProcessDetail = true;
	private String listPath = null;
	
    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doStartTag:: invocato");
        if(pageContext == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,
                "ProcessInstanceListTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } 
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } catch(Exception ex) {
            TracerSingleton.log(
            		Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        if(httpRequest == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        }
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if(responseContainer == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag:: httpRequest nullo");
            throw new JspException("responseContainer nullo");
        }    
        StringBuffer output = new StringBuffer(); 
       
        SourceBean listBean = (SourceBean)responseContainer.getServiceResponse().getAttribute(getListPath());
		String process = (String)listBean.getAttribute(Constants.LBL_PROCESS_ID);
        List processList = listBean.getAttributeAsList("ITEM");
		TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doStartTag:: processList.size: " + processList.size());
		TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doStartTag:: serviceResponse is:" + responseContainer.getServiceResponse());
		
		if(processList.size() > 0) {
			SourceBean processInstance;
			Long processInstanceId, processId;
			int indexProcess;
			for(indexProcess=0; indexProcess<processList.size();indexProcess++) {
				try {
					processInstance = (SourceBean)processList.get(indexProcess);
					processInstanceId = (Long)processInstance.getAttribute("id"); 
					processId = (Long)processInstance.getAttribute("idProcesso");
					output.append("<tr>\n");			
					output.append("<td>" + processInstanceId + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("processo") + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("iter") + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("iterIstanza") + "</td>\n");
					if (isAdvancedSearch())
						output.append("<td>" + (String)processInstance.getAttribute("owner") + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("dataInizio") + "</td>\n");
					output.append("<td>" + (String)processInstance.getAttribute("dataFine") + "</td>\n");
					
					int stato = ((Integer)processInstance.getAttribute("stato")).intValue();
					if(stato == ServiceAssemblyStateConstants.SA_ACTIVE)
						output.append("<td><img src=\"" + httpRequest.getContextPath() + "/images/console/wait.gif\" alt=\"Running\"/>");
					else if(stato == ServiceAssemblyStateConstants.SA_DONE) 
						output.append("<td><img src=\"" + httpRequest.getContextPath() + "/images/console/executed.gif\" alt=\"Executed\"/>");	
					else
						output.append("<td><img src=\"" + httpRequest.getContextPath() + "/images/console/fault.gif\" alt=\"Faulted\"/>");
	
					if ("TRUE".equals(processInstance.getAttribute("restarted"))) {
						output.append("<img src=\"" + httpRequest.getContextPath() + "/images/console/lightning.png\" alt=\"Restarted\"/>");	
					}
					output.append("</td>\n");	
					
					if (isShowProcessDetail() || (!isShowProcessDetail() && process == null)){
						String advancedSearch = isAdvancedSearch()? "TRUE" : "FALSE";
						output.append("<td><a href=\"#\" onclick=\"showProcessInstanceGraph('" + processInstanceId + "', '" + processId + "','" + advancedSearch +"');return false;\"><img src=\"" + httpRequest.getContextPath() + "/images/console/graph.gif\"/></a></td>\n");
					} else 
						output.append("<td></td>");
					output.append("</tr>\n");
				} catch(ClassCastException cce) {
					TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag::", cce);
				}
			}
		} else {//generate a row empty
			output.append("<tr>\n");			
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			if (isAdvancedSearch())
				output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>\n");
			output.append("<td></td>");
			output.append("</tr>\n");
		}
		
        try {
            pageContext.getOut().print(output.toString());
        } catch(Exception ex) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        return SKIP_BODY;
    }
    
	public boolean isAdvancedSearch() {
		return _advancedSearch;
	}

	public void setAdvancedSearch(boolean advancedSearch) {
		_advancedSearch = advancedSearch;
	}

	
	public boolean isShowProcessDetail() {
		return _showProcessDetail;
	}

	public void setShowProcessDetail(boolean processDetail) {
		_showProcessDetail = processDetail;
	}
	
	public String getListPath() {
		return listPath;
	}

	public void setListPath(String listPath) {
		this.listPath = listPath;
	}

	/**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "ProcessInstanceListTag::doEndTag:: invocato");
        return super.doEndTag();
    } 

}
