/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.tag;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import it.eng.console.util.Constants;
import it.eng.serviceregistry.IServiceRegistry;
import it.eng.serviceregistry.JAXRServiceRegistryImpl;
import it.eng.serviceregistry.ServiceRegistryException;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;
import javax.xml.registry.JAXRException;
import javax.xml.registry.infomodel.ClassificationScheme;

public class TaxonomyTreeTag extends TagSupport {
	protected String _treeElementId = null;
	protected String _actionName = null;
	
    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "TaxonomyTreeTag::doStartTag:: invocato");
        if(pageContext == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,
                "TaxonomyTreeTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } 
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } catch(Exception ex) {
            TracerSingleton.log(
            		Constants.project, TracerSingleton.CRITICAL, "TaxonomyTreeTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        if(httpRequest == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "TaxonomyTreeTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        } 
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if(responseContainer == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "TaxonomyTreeTag::doStartTag:: httpRequest nullo");
            throw new JspException("responseContainer nullo");
        }        
        
    	SourceBean serviceResponse = responseContainer.getServiceResponse();
    	String idSelectedNodeFromResponse = (String)serviceResponse.getAttribute("LIST.idSelectedNode");
    	int idSelectedNode = -1;
    	if (idSelectedNodeFromResponse!=null) 			
    		idSelectedNode = Integer.parseInt(idSelectedNodeFromResponse);
                
        StringBuffer output = new StringBuffer(); 
        output.append("<script>\n");
        IServiceRegistry serviceRegistry = null;
        try {
	        serviceRegistry = new JAXRServiceRegistryImpl();
	        serviceRegistry.initRegistry();
			List taxonomyList = serviceRegistry.getAllClassificationScheme();
	               	  
	        output.append("function initTree() {\n");
	        output.append("taxonomyTree = new dTree('taxonomyTree');\n");
	        output.append("taxonomyTree.add(0,-1,'" + getLabel("selectTaxonomy")+ "','" + httpRequest.getContextPath() + "/" + _actionName + ".action','','','" + httpRequest.getContextPath() +"/images/console/graph.gif' );\n");

	        ClassificationScheme taxonomy = null;
	        String taxonomyName = null;
	        String taxonomyValue = null;
	        Collection classifications = null;
	        Iterator classificationsIt = null;
	        final int indexRoot = 0;
	        int indexParent;
	        int indexNodes = 1;
	        for (int indexTaxonomy=0; indexTaxonomy<taxonomyList.size(); indexTaxonomy++) {
	        	taxonomy = (ClassificationScheme)taxonomyList.get(indexTaxonomy);
	        	try {
	        		taxonomyName = taxonomy.getName().getValue();
	        		output.append("taxonomyTree.add(" + indexNodes + ", " + indexRoot + ", '" + taxonomyName + "');\n");
	        		indexParent = indexNodes;
	        		indexNodes++;
	        		classifications = serviceRegistry.getClassificationSchemeValues(taxonomyName);
	        		classificationsIt = classifications.iterator();
	        		while(classificationsIt.hasNext()) {
	        			taxonomyValue = (String)classificationsIt.next();
	        			output.append("taxonomyTree.add(" + indexNodes 
	        						+ ", " + indexParent 
	        						+ ", '" + taxonomyValue 
	        						+ "', '" + httpRequest.getContextPath() + "/" + _actionName + ".action?taxonomyName=" + taxonomyName 
	        						+ "&taxonomyValue=" + taxonomyValue + "&idSelectedNode=" + indexNodes + "');\n");
	        			indexNodes++;
	        		}
	        	} catch(JAXRException jax) {
	        		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "TaxonomyTreeTag::doStartTag::", jax);
	                throw new JspException(jax.getMessage());
	        	} 
	        	
	        }
	        output.append("document.getElementById('" + _treeElementId + "').innerHTML = taxonomyTree;");
	        if(idSelectedNode!=-1)
	        	output.append("taxonomyTree.openTo(" + idSelectedNode + ",true);");
	        	
        } catch(ServiceRegistryException sre) {
        	TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "TaxonomyTreeTag::doStartTag::", sre);
            throw new JspException(sre.getMessage());
        } finally {
			if(serviceRegistry != null)
				serviceRegistry.finalizeRegistry();
		}
        
        output.append("}\n");        
        output.append("</script>\n");
        try {
            pageContext.getOut().print(output.toString());
        } catch (Exception ex) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "TaxonomyTreeTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        return SKIP_BODY;
    } 
    
    private String getLabel(String id) {
    	return MessageBundle.getMessage(id, "properties.spagic");
    }
    
    public String getTreeElementId() {
		return _treeElementId;
	}

	public void setTreeElementId(String treeElementId) {
		_treeElementId = treeElementId;
	}
	
	public String getActionName() {
		return _actionName;
	}

	public void setActionName(String actionName) {
		_actionName = actionName;
	}

	/**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "TaxonomyTreeTag::doEndTag:: invocato");
        return super.doEndTag();
    } // public int doEndTag() throws JspException
}
