/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.jmx.bo;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import it.eng.console.util.Constants;
import it.eng.jmx.beans.SummaryBean;
import it.eng.jmx.util.JmxConnector;
import it.eng.spago.tracing.TracerSingleton;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;

public class JmxBO {

	public static SummaryBean getSummary(String idServer) {
		SummaryBean summary = new SummaryBean();
		try {
			JmxConnector connector = (new JmxConnector(idServer));
			MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
			ObjectName thread = new ObjectName("java.lang:type=Threading");
			ObjectName classes = new ObjectName("java.lang:type=ClassLoading");
		    summary.setPeak( ((Integer)connection.getAttribute(thread,"PeakThreadCount")).intValue()); 
		    summary.setDaemonThreads(((Integer)connection.getAttribute(thread,"DaemonThreadCount")).intValue());    
		    summary.setTotalThreads(((Long)connection.getAttribute(thread,"TotalStartedThreadCount")).longValue());
		    summary.setLiveThreads(((Integer)connection.getAttribute(thread,"ThreadCount")).intValue());
		    summary.setCurrentClassesLoaded(((Integer)connection.getAttribute(classes,"LoadedClassCount")).intValue());
		    summary.setTotalClassesLoaded(((Long)connection.getAttribute(classes,"TotalLoadedClassCount")).longValue());
		    connector.close();
		} catch (Exception ex) {
           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getSummary: ", ex);
        } 
        return summary;
	}
	
	public static String[] getQueueAttributes(String idServer) {
		return getQueueAttributes(idServer,"org.apache.activemq:BrokerName=localhost,Type=Queue,*");
	}
	
	public static String[] getTopicAttributes(String idServer) {
		return getQueueAttributes(idServer,"org.apache.activemq:BrokerName=localhost,Type=Topic,*");
	}
	
	public static String[] getQueueAttributes(String idServer, String labelObjectName) {
		String[] attributesName = null;
		Set queueObjectNameList = null;
		try {
			JmxConnector connector = (new JmxConnector(idServer));
			MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
			queueObjectNameList = connection.queryNames(new ObjectName("org.apache.activemq:BrokerName=localhost,Type=Queue,*"),null);
			ObjectName objName = (ObjectName) queueObjectNameList.iterator().next();
			MBeanAttributeInfo[] infoMBeanAttributes = connection.getMBeanInfo(objName).getAttributes();
			attributesName = new String[infoMBeanAttributes.length];
			int indexAttribute;
			for (indexAttribute = 0; indexAttribute < infoMBeanAttributes.length; indexAttribute++) 
				attributesName[indexAttribute] = infoMBeanAttributes[indexAttribute].getName();
			
			connector.close();
		} catch (Exception ex) {
           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getQueue(String idServer, String labelObjectName): ", ex);
        } 
		return attributesName;
	}
	
	public static List getQueue(String idServer) throws Exception {
        return getQueue(idServer, "org.apache.activemq:BrokerName=localhost,Type=Queue,*");
	}

	public static List getTopic(String idServer) throws Exception {
		return getQueue(idServer, "org.apache.activemq:BrokerName=localhost,Type=Topic,*");
	}
	
	public static List getQueue(String idServer, String labelObjectName) throws Exception {
		String[] attributes = getQueueAttributes(idServer, labelObjectName);
		List queueValueList = new ArrayList();
		
		Set queueObjectNameList = null;
		try {
			JmxConnector connector = (new JmxConnector(idServer));
			MBeanServerConnection connection = connector.getJmxConnector().getMBeanServerConnection();
			queueObjectNameList = connection.queryNames(new ObjectName(labelObjectName),null);
			ObjectName objName = null;
			for (Iterator i = queueObjectNameList.iterator(); i.hasNext(); ) {
				objName = (ObjectName) i.next();
				queueValueList.add(connection.getAttributes(objName, attributes));
			}
	        connector.close();
		} catch (Exception ex) {
           TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxBO::getQueue(String idServer, String labelObjectName): ", ex);
           throw ex;
        } 
        return queueValueList;
	}
}
