/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.jmx.util;

import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;

import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

public class JmxConnector {
	private JMXConnector _jmxConnector = null;

	public JmxConnector(String idServer) 
			throws Exception {
		ConfigSingleton config = ConfigSingleton.getInstance();
		SourceBean serverConfig = (SourceBean)config.getFilteredSourceBeanAttribute("SERVERS.SERVER", "id", idServer);
		String user = (String)serverConfig.getAttribute("user");
		String psw = (String)serverConfig.getAttribute("psw");
		String jmxUrl = (String)serverConfig.getAttribute("jmxUrl");
		try {
			HashMap env = new HashMap();
	        String[] credentials = new String[] { user , psw};
	        env.put("jmx.remote.credentials", credentials);
	        JMXServiceURL url = new JMXServiceURL(jmxUrl);
	        _jmxConnector = JMXConnectorFactory.connect(url, env);
		} catch (MalformedURLException ue) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxServerConnection::getConnection: ", ue);
			throw ue;
	    } catch (IOException ioe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxServerConnection::getConnection: ", ioe);
			throw ioe;
	    }
	}

	public JMXConnector getJmxConnector() {
		return _jmxConnector;
	}

	public void setJmxConnector(JMXConnector connector) {
		_jmxConnector = connector;
	}
	
	public void close()throws Exception {
		try {
			_jmxConnector.close();
		} catch (IOException ioe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "JmxServerConnection::getConnection: ", ioe);
			throw ioe;
	    }
	}
}
